$|++;
use strict;
use Test::More;

# Add skips here
BEGIN {
  eval { use Pod::Usage };
  if ( $@ ) {
    plan skip_all => "Pod::Usage must be installed to run these tests";
  }
}

plan tests => 11;

use t::common qw( new_fh );
use File::Spec;

my $PROG = File::Spec->catfile( qw( utils upgrade_db.pl ) );

my $short = get_pod( $PROG, 0 );
my $long = get_pod( $PROG, 1 );

is( run_prog( $PROG ), "Missing required parameters.\n$long", "Failed no params" );
is( run_prog( $PROG, '-input foo' ), "Missing required parameters.\n$long", "Failed only -input" );
is( run_prog( $PROG, '-output foo' ), "Missing required parameters.\n$long", "Failed only -output" );
is(
    run_prog( $PROG, '-input foo', '-output foo' ),
    "Cannot use the same filename for both input and output.\n$short",
    "Failed same name",
);

is(
    run_prog( $PROG, '-input foo', '-output bar' ),
    "'foo' is not a file.\n$short",
    "Failed input does not exist",
);

my (undef, $input_filename) = new_fh();
my (undef, $output_filename) = new_fh();

is(
    run_prog( $PROG, "-input $input_filename", "-output $output_filename" ),
    "'$input_filename' is not a DBM::Deep file.\n$short",
    "Input is not a DBM::Deep file",
);

# All files are of the form:
#   $db->{foo} = [ 1 .. 3 ];

my @versions = (
    '0-983',
    '0-99_04',
    '1-0000',
);

foreach my $input_version ( @versions ) {
    my $input_filename = File::Spec->catfile( qw( t etc ), "db-$input_version" );

    foreach my $output_version ( @versions ) {
        (my $v = $output_version) =~ s/-/./g;
        my $output = run_prog(
            $PROG,
            "-input $input_filename",
            "-output $output_filename",
            "-version $v",
        );

        if ( $input_version =~ /_/ ) {
            is(
                $output, "'$input_filename' is a dev release and not supported.\n$short",
                "Input file is a dev release - not supported",
            );

            next;
        }

        if ( $output_version =~ /_/ ) {
            is(
                $output, "-version '$v' is a dev release and not supported.\n$short",
                "Output version is a dev release - not supported",
            );

            next;
        }
    }
}

################################################################################

sub run_prog {
  #print "Launching '@_'\n";
  #XXX This needs to be made OS-portable
  open( my $fh, '-|', "@_ 2>&1" ) or die "Cannot launch '@_': $!\n";
  return join '', <$fh>;
}

sub get_pod {
  my ($p,$v) = @_;
  #XXX This needs retro'ed to 5.6.0
  open( my $fh, '>', \my $pod ) or die "Cannot open in-memory filehandle: $!\n";
  pod2usage({
    -input   => $p,
    -output  => $fh,
    -verbose => $v,
    -exitval => 'NOEXIT',
  });
  return $pod;
}
